"""Information about the road the object is on"""

# Copyright © 2024 understandAI GmbH
#
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files
# (the “Software”), to deal in the Software without restriction, including without limitation the rights to use, copy, modify,
# merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


from dataclasses import dataclass, field
from enum import Enum
from typing import Literal

from apischema.metadata import required
from uai_openlabel import (
    NumberData,
    TextData,
    no_default,
)


@dataclass
class Road__SpeedLimit(NumberData):
    """Applicable speed limit for the vehicle at its current position in (m/s).
    0 indicates no speed limit (i.e. unlimited)."""

    val: float = field(default_factory=lambda: no_default(field="Road__SpeedLimit.val"), metadata=required)
    """Speed limit value in (m/s)"""

    name: Literal["road/speed_limit"] = field(default="road/speed_limit")
    """Is always 'road/speed_limit'"""


class Road__ClassificationValue(str, Enum):
    """Possible values of 'track_section'"""

    STRAIGHT = "straight"
    """A straight road section"""

    CURVE = "curve"
    """A curving road section"""

    T_INTERSECTION = "t_intersection"
    """A T intersection"""

    X_INTERSECTION = "x_intersection"
    """A crossing"""

    ROUNDABOUT = "roundabout"
    """A roundabout"""

    PARKING_AREA = "parking_area"
    """A parking lot or area"""

    PROPERTY_ENTRANCE = "property_entrance"
    """A property entrance"""


@dataclass
class Road__Classification(TextData):
    """The classification of the current road section"""

    val: Road__ClassificationValue = field(
        default_factory=lambda: no_default(field="Road__Classification.val"), metadata=required
    )
    """See `Road__ClassificationValue` for possible values."""

    name: Literal["road/classification"] = field(default="road/classification")
    """Is always 'road/classification'"""


@dataclass
class Road__NumberLanes__Left__Legal(NumberData):
    """Number of lanes left of vehicle's position that the vehicle is legally permitted to use.

    A value of -1 means that this attribute is not applicable.
    A lane change occurs when the center of the bounding box crosses the lane boundary.
    """

    val: int = field(default_factory=lambda: no_default(field="Road__NumberLanes__Left__Legal.val"), metadata=required)
    """Number of lanes left of vehicle's position that the vehicle is legally permitted to use."""

    name: Literal["road/number_lanes/left/legal"] = field(default="road/number_lanes/left/legal")
    """Is always 'road/number_lanes/left/legal'"""


@dataclass
class Road__NumberLanes__Left__Physical(NumberData):
    """
    Number of lanes left of vehicle's position that the vehicle can physically reach, excluding its current lane.
    This includes lanes the vehicle is not allowed to drive on.

    A value of -1 means that this attribute is not applicable.
    A lane change occurs when the center of the bounding box crosses the lane boundary.
    """

    val: int = field(default_factory=lambda: no_default(field="Road__NumberLanes__Left__Physical.val"), metadata=required)
    """Number of lanes left of vehicle's position that the vehicle can physically reach, excluding its current lane."""

    name: Literal["road/number_lanes/left/physical"] = field(default="road/number_lanes/left/physical")
    """Is always 'road/number_lanes/left/physical'"""


@dataclass
class Road__NumberLanes__Right__Legal(NumberData):
    """Number of lanes right of vehicle's position that the vehicle is legally permitted to use.

    A value of -1 means that this attribute is not applicable.
    A lane change occurs when the center of the bounding box crosses the lane boundary.
    """

    val: int = field(default_factory=lambda: no_default(field="Road__NumberLanes__Right__Legal.val"), metadata=required)
    """Number of lanes right of vehicle's position that the vehicle is legally permitted to use."""

    name: Literal["road/number_lanes/right/legal"] = field(default="road/number_lanes/right/legal")
    """Is always 'road/number_lanes/right/legal'"""


@dataclass
class Road__NumberLanes__Right__Physical(NumberData):
    """
    Number of lanes right of vehicle's position that the vehicle can physically reach, excluding its current lane.
    This includes lanes the vehicle is not allowed to drive on.

    A value of -1 means that this attribute is not applicable.
    A lane change occurs when the center of the bounding box crosses the lane boundary.
    """

    val: int = field(default_factory=lambda: no_default(field="Road__NumberLanes__Right__Physical.val"), metadata=required)
    """Integer number of lanes"""

    name: Literal["road/number_lanes/right/physical"] = field(default="road/number_lanes/right/physical")
    """Is always 'road/number_lanes/right/physical'"""



# Copyright © 2024 understandAI GmbH
#
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files
# (the “Software”), to deal in the Software without restriction, including without limitation the rights to use, copy, modify,
# merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

from dataclasses import dataclass, field
from typing import Optional, Sequence, Union

from apischema.metadata import required
from uai_openlabel import (
    URI,
    Context,
    ContextUid,
    CoordinateSystem,
    CoordinateSystemUid,
    EventUid,
    FrameInterval,
    Object,
    ObjectUid,
    OntologyUid,
    ResourceUid,
    Stream,
    StreamUid,
    Uid,
    no_default,
)
from uai_openlabel import (
    DetailedOntology as Ontology,
)
from uai_openlabel import (
    OpenLabel as BaseOpenLabel,
)

from aveas_openlabel.event import Event
from aveas_openlabel.frame import Frame
from aveas_openlabel.metadata import AcquisitionMethod, Metadata, RightOfUse

__all__: list[str] = []


@dataclass
class AveasOpenLabel(BaseOpenLabel):
    """The AVEAS OpenLABEL class.

    The AVEAS OpenLABEL specification is a subset of the standard ASAM OpenLABEL standard.
    Any OpenLABEL file generated according to the AVEAS OpenLABEL specification is always a valid ASAM OpenLABEL file.
    """

    metadata: Metadata = field(default_factory=lambda: no_default(field="AveasOpenLabel.metadata"), metadata=required)
    """
        This JSON object contains metadata about the annotation file itself (see `Metadata`). 
    """

    # actions: NOT SPECIFIED FOR AVEAS OPENLABEL

    contexts: Optional[dict[ContextUid, Context]] = field(default=None)
    """Information on the contexts of the scenario
    """

    coordinate_systems: Optional[dict[CoordinateSystemUid, CoordinateSystem]] = field(default=None)
    """
        This field contains OpenLABEL coordinate systems. 
        Coordinate system keys can be any string, for example, a friendly coordinate system name.
    """

    events: Optional[dict[EventUid, Event]] = field(default=None)
    """
        This field contains the events that occurred in the scenario. 
    """

    frame_intervals: Optional[Sequence[FrameInterval]] = field(default=None)
    """
        The interval of frames contained in this scenario. 
    """

    frames: Optional[dict[Uid, Frame]] = field(default=None)
    """
        This field contains the frames that contain the dynamic attributes of the classifications found in `aveas_openlabel.classifications`.
        The keys of this dict are frame IDs. 
    """

    objects: Optional[dict[ObjectUid, Object]] = field(default=None)
    """
        This field contains the static attributes of the classifications found in `aveas_openlabel.classifications`. 
        Each unique object in a scenario is distinguishable by a unique identifier.
    """

    ontologies: Optional[dict[OntologyUid, Union[URI, Ontology]]] = field(default=None)
    """
         This field contains OpenLABEL ontologies. 
         Ontology keys are strings containing numerical UIDs or 32 bytes UUIDs.
         Ontology values may be strings, such as URLs. 
         containing a URI string and optional Sequences of included and excluded terms.
    """

    # relations: NOT SPECIFIED FOR AVEAS OPENLABEL

    resources: Optional[dict[ResourceUid, URI]] = field(default=None)
    """
        This field contains OpenLABEL resources. 
        Resource keys are strings containing numerical UIDs or 32 bytes UUIDs.
        Resource values are strings that describe an external resource, for example, file name, URLs, that may be
        used to link data of the OpenLABEL annotation content with external existing content.
    """

    streams: Optional[dict[StreamUid, Stream]] = field(default=None)
    """
        Streams (cameras, lidars) that were used to record the data. 
        Information about the sensors, such as intrinsic calibration information, is contained in this field. 
    """

    # tags: NOT SPECIFIED FOR AVEAS OPENLABEL

    @classmethod
    def minimum_example(cls: type["AveasOpenLabel"]) -> "AveasOpenLabel":
        return cls(
            metadata=Metadata(
                right_of_use=RightOfUse.RESEARCH_ONLY,
                acquisition_method=AcquisitionMethod.IN_VEHICLE,
                acquisition_partner="foo bar institute",
                acquisition_date="2000-01-01T01:01:01.001Z",
                projection_string="example projection string",
                threshold_gttc=1.0,
                threshold_pret=1.0,
                threshold_thw=3.0,
            )
        )



"""Size and center of gravity"""

# Copyright © 2024 understandAI GmbH
#
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files
# (the “Software”), to deal in the Software without restriction, including without limitation the rights to use, copy, modify,
# merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

from dataclasses import dataclass, field
from typing import Literal

from apischema.metadata import required
from uai_openlabel import VectorData, no_default


@dataclass
class Dimensions__Size(VectorData):  # TODO: Koordinatensystem wie spezifizieren?
    """Size of this object"""

    val: tuple[float, float, float] = field(default_factory=lambda: no_default(field="Dimensions__Size.val"), metadata=required)
    """The (x, y, z) extent of the object in (m)."""

    name: Literal["dimensions/size"] = field(default="dimensions/size")
    """Is always 'dimensions/size'"""


@dataclass
class Dimensions__Size__UStdDev(VectorData):
    """Uncertainty for `Dimensions__Size` as standard deviation around the parent indicated value."""

    val: tuple[float, float, float] = field(
        default_factory=lambda: no_default(field="Dimensions__Size__UStdDev"), metadata=required
    )
    """Standard deviation around the parent indicated value. """

    name: Literal["dimensions/size/ustddev"] = field(default="dimensions/size/ustddev")
    """Is always 'dimensions/size/ustddev'."""


@dataclass
class Dimensions__CenterOfGravity(VectorData):  # TODO: Koordinatensystem wie spezifizieren?
    """Center of gravity of this object, relative to the center of the bounding box."""

    val: tuple[float, float, float] = field(
        default_factory=lambda: no_default(field="Dimensions__CenterOfGravity.val"), metadata=required
    )
    """The (x, y, z) offset of the center of gravity relative to the center of the bounding box. """

    name: Literal["dimensions/center_of_gravity"] = field(default="dimensions/center_of_gravity")
    """Is always 'dimensions/center_of_gravity'"""


@dataclass
class Dimensions__CenterOfGravity__UStdDev(VectorData):
    """Uncertainty for `Dimensions__CenterOfGravity` as standard deviation around the parent indicated value."""

    val: tuple[float, float, float] = field(
        default_factory=lambda: no_default(field="Dimensions__CenterOfGravity__UStdDev.val"), metadata=required
    )
    """Standard deviation around the parent indicated value. """

    name: Literal["dimensions/center_of_gravity/ustddev"] = field(default="dimensions/center_of_gravity/ustddev")
    """Is always 'dimensions/center_of_gravity/ustddev'."""



"""Implementation of attributes in `ScenarioContextData`

"""

# Copyright © 2024 understandAI GmbH
#
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files
# (the “Software”), to deal in the Software without restriction, including without limitation the rights to use, copy, modify,
# merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


from dataclasses import dataclass, field
from typing import Literal

from apischema.metadata import required
from uai_openlabel import (
    BooleanData,
    NumberData,
    TextData,
    VectorData,
    no_default,
)


@dataclass
class Scenario__IsStaged(BooleanData):
    """True iff the scenario contains staged (i.e. previously defined) behavior. This can include simulator studies
    where other vehicles drive on specifically set trajectories, or real-world scenarios when one or several vehicles
    are driven by specifically instructed operators. It does not include simulations where vehicle behavior is
    stochastically determined by a behavior model.
    """

    val: bool = field(default_factory=lambda: no_default(field="Scenario__IsStaged.val"), metadata=required)
    """True iff the scenario contains staged (i.e. previously defined) behavior."""

    name: Literal["scenario/is_staged"] = field(default="scenario/is_staged")
    """Is always 'scenario/is_staged'"""


@dataclass
class Scenario__IsBiased(BooleanData):
    """True iff the scenario likely does NOT represent purely natural conditions due to recording- or study-specific
    influence, for example due to a conspicuous recording vehicle affecting the driving behavior of other traffic
    participants, or due to unusual staged behavior (cf. `Scenario__IsStaged`).
    """

    val: bool = field(default_factory=lambda: no_default(field="Scenario__IsBiased.val"), metadata=required)
    """Value indicating if scenario is biased or not."""

    name: Literal["scenario/is_biased"] = field(default="scenario/is_biased")
    """Is always 'scenario/is_biased'"""


@dataclass
class Scenario__IsSampled(BooleanData):
    """True iff the scenario has been sampled from a real scenario or not. See also `Scenario__IsSampled__ReferenceToSourceScenario`."""

    val: bool = field(default_factory=lambda: no_default(field="Scenario__IsSampled.val"), metadata=required)
    """Value indicating if scenario is a sampled one or not."""

    name: Literal["scenario/is_sampled"] = field(default="scenario/is_sampled")
    """Is always 'scenario/is_sampled'"""


@dataclass
class Scenario__IsSampled__ReferenceToSourceScenario(TextData):
    """Specifying the scenario ID of the source scenario the scenario was sampled from. Only required if `Scenario__IsSampled` is True."""

    val: str = field(
        default_factory=lambda: no_default(field="Scenario__IsSampled__ReferenceToSourceScenario.val"), metadata=required
    )
    """The scenario ID of the source scenario."""

    name: Literal["scenario/is_sampled/reference_to_source_scenario"] = field(
        default="scenario/is_sampled/reference_to_source_scenario"
    )
    """Is always 'scenario/is_sampled/reference_to_source_scenario'"""


@dataclass
class Scenario__MinimumVehicleDistanceS(NumberData):
    """The minimum distance between a moving vehicle and another traffic participant occurring in the scenario in the direction of the road's reference line."""

    val: float = field(default_factory=lambda: no_default(field="Scenario__MinimumVehicleDistanceS.val"), metadata=required)
    """The value of the minimum distance in direction of the road's reference line."""

    name: Literal["scenario/minimum_vehicle_distance_s"] = field(default="scenario/minimum_vehicle_distance_s")
    """Is always 'scenario/minimum_vehicle_distance_s'"""


@dataclass
class Scenario__MinimumVehicleDistanceS__UStdDev(NumberData):
    """Uncertainty for `Scenario__MinimumVehicleDistanceS` as standard deviation around the parent indicated value."""

    val: float = field(default_factory=lambda: no_default(field="FrameMinimumVehicleDistanceS__UStdDev.val"), metadata=required)
    """Standard deviation around the parent indicated value. """

    name: Literal["scenario/minimum_vehicle_distance_s/ustddev"] = field(default="scenario/minimum_vehicle_distance_s/ustddev")
    """Is always 'scenario/minimum_vehicle_distance_s/ustddev'."""


@dataclass
class Scenario__MinimumVehicleDistanceS__Frame(TextData):
    """Specifying the frame ID for which Scenario__MinimumVehicleDistanceS occurs."""

    val: str = field(
        default_factory=lambda: no_default(field="Scenario__MinimumVehicleDistanceS__Frame.val"), metadata=required
    )
    """The frame ID for which Scenario__MinimumVehicleDistanceS occurs."""

    name: Literal["scenario/minimum_vehicle_distance_s/frame"] = field(default="scenario/minimum_vehicle_distance_s/frame")
    """Is always 'scenario/minimum_vehicle_distance_s/frame'"""


@dataclass
class Scenario__ContainsHighway(BooleanData):
    """Indicating if the scenario is (partially) taking place on highways."""

    val: bool = field(default_factory=lambda: no_default(field="Scenario__ContainsHighway.val"), metadata=required)
    """True iff the scenario is (partially) taking place on highways."""

    name: Literal["scenario/contains_highway"] = field(default="scenario/contains_highway")
    """Is always 'scenario/contains_highway'"""


@dataclass
class Scenario__ContainsUrban(BooleanData):
    """Indicating if the scenario is (partially) taking place in urban environments."""

    val: bool = field(default_factory=lambda: no_default(field="Scenario__ContainsUrban.val"), metadata=required)
    """True iff the scenario is (partially) taking place in urban environments."""

    name: Literal["scenario/contains_urban"] = field(default="scenario/contains_urban")
    """Is always 'scenario/contains_urban'"""


@dataclass
class Scenario__ContainsRural(BooleanData):
    """Indicating if the scenario is (partially) taking place in rural environments."""

    val: bool = field(default_factory=lambda: no_default(field="Scenario__ContainsRural.val"), metadata=required)
    """True iff the scenario is (partially) taking place in rural environments."""

    name: Literal["scenario/contains_rural"] = field(default="scenario/contains_rural")
    """Is always 'scenario/contains_rural'"""


@dataclass
class Scenario__Course(VectorData):
    """Describing the course of a scenario using a sequence of road names in case of in-vehicle or aerial data acquisition or the scenario location using a single road/intersection name in case of stationary data acquisition. The aim of this field is to quickly give the user an idea of where the scenario takes place."""

    val: list[str] = field(default_factory=lambda: no_default(field="Scenario__Course.val"), metadata=required)
    """List of road names for in-vehicle/aerial acquisition, road/intersection name for stationary acquisition."""

    name: Literal["scenario/course"] = field(default="scenario/course")
    """Is always 'scenario/course'"""


@dataclass
class Scenario__MinimumVehicleSpeed(NumberData):
    """Minimum speed of moving vehicles in the scenario.

    Indicating the minimum velocity of all vehicles involved in the scenario that have a velocity of greater than
    zero at least once during the scenario.
    """

    val: float = field(default_factory=lambda: no_default(field="Scenario__MinimumVehicleSpeed.val"), metadata=required)
    """The value of the minimum vehicle velocity."""

    name: Literal["scenario/minimum_vehicle_speed"] = field(default="scenario/minimum_vehicle_speed")
    """Is always 'scenario/minimum_vehicle_speed'"""


@dataclass
class Scenario__MinimumVehicleSpeed__UStdDev(NumberData):
    """Uncertainty for `Scenario__MinimumVehicleSpeed` as standard deviation around the parent indicated value."""

    val: float = field(
        default_factory=lambda: no_default(field="Scenario__MinimumVehicleSpeed__UStdDev.val"), metadata=required
    )
    """Standard deviation around the parent indicated value."""

    name: Literal["scenario/minimum_vehicle_speed/ustddev"] = field(default="scenario/minimum_vehicle_speed/ustddev")
    """Is always 'scenario/minimum_vehicle_speed/ustddev'."""


@dataclass
class Scenario__MinimumVehicleSpeed__Frame(TextData):
    """Specifying the frame ID for which Scenario__MinimumVehicleSpeed occurs."""

    val: str = field(default_factory=lambda: no_default(field="Scenario__MinimumVehicleSpeed__Frame.val"), metadata=required)
    """The frame ID for which Scenario__MinimumVehicleSpeed occurs."""

    name: Literal["scenario/minimum_vehicle_speed/frame"] = field(default="scenario/minimum_vehicle_speed/frame")
    """Is always 'scenario/minimum_vehicle_speed/frame'"""


@dataclass
class Scenario__MaximumVehicleSpeed(NumberData):
    """Indicating the maximum velocity of all vehicles involved in this scenario."""

    val: float = field(default_factory=lambda: no_default(field="Scenario__MaximumVehicleSpeed.val"), metadata=required)
    """The value of the maximum vehicle velocity."""

    name: Literal["scenario/maximum_vehicle_speed"] = field(default="scenario/maximum_vehicle_speed")
    """Is always 'scenario/maximum_vehicle_speed'"""


@dataclass
class Scenario__MaximumVehicleSpeed__UStdDev(NumberData):
    """Uncertainty for `Scenario__MaximumVehicleSpeed` as standard deviation around the parent indicated value."""

    val: float = field(default_factory=lambda: no_default(field="Scenario__MaximumVehicleSpeed.val"), metadata=required)
    """Standard deviation around the parent indicated value. """

    name: Literal["scenario/maximum_vehicle_speed/ustddev"] = field(default="scenario/maximum_vehicle_speed/ustddev")
    """Is always 'scenario/maximum_vehicle_speed/ustddev'."""


@dataclass
class Scenario__MaximumVehicleSpeed__Frame(TextData):
    """Specifying the frame ID for which Scenario__MaximumVehicleSpeed occurs."""

    val: str = field(default_factory=lambda: no_default(field="Scenario__MaximumVehicleSpeed__Frame.val"), metadata=required)
    """The frame ID for which Scenario__MaximumVehicleSpeed occurs."""

    name: Literal["scenario/maximum_vehicle_speed/frame"] = field(default="scenario/maximum_vehicle_speed/frame")
    """Is always 'scenario/maximum_vehicle_speed/frame'"""


@dataclass
class Scenario__Start__Location(TextData):
    """Describes the location where the scenario started in human-readable format, e.g. “on-ramp Moensheim”."""

    val: str = field(default_factory=lambda: no_default(field="Scenario__Start__Location.val"), metadata=required)
    """The start location of the scenario."""

    name: Literal["scenario/start/location"] = field(default="scenario/start/location")
    """Is always 'scenario/start/location'"""


@dataclass
class Scenario__Start__Coordinates(VectorData):
    """
    Contains the coordinates of the start location of the scenario using the coordinate system also used within the OpenDRIVE file.
    In the case of aerial data acquisition, this refers to the initial position of the plane.
    In the case of in-vehicle data acquisition, to the initial position of the recording vehicle.
    """

    val: tuple[float, float, float] = field(
        default_factory=lambda: no_default(field="Scenario__Start__Coordinates.val"), metadata=required
    )
    """The coordinates of the start position in the order x, y, z."""

    name: Literal["scenario/start/coordinates"] = field(default="scenario/start/coordinates")
    """Is always 'scenario/start/coordinates'"""


@dataclass
class Scenario__End__Location(TextData):
    """Describes the location where the scenario ended in human-readable format, e.g. “on-ramp Moensheim”."""

    val: str = field(default_factory=lambda: no_default(field="Scenario__End__Location.val"), metadata=required)
    """The end location of the scenario."""

    name: Literal["scenario_end/location"] = field(default="scenario_end/location")
    """Is always 'scenario/end/location'"""


@dataclass
class Scenario__End__Coordinates(VectorData):
    """
    Contains the coordinates of the end location of the scenario using the coordinate system also used within the OpenDRIVE file.
    In the case of aerial data acquisition, this refers to the final position of the plane.
    In the case of in-vehicle data acquisition, to the final position of the recording.
    """

    val: tuple[float, float, float] = field(
        default_factory=lambda: no_default(field="Scenario__End__Coordinates.val"), metadata=required
    )
    """The coordinates of the end position in the order x, y, z."""

    name: Literal["scenario/end/coordinates"] = field(default="scenario/end/coordinates")
    """Is always 'scenario/end/coordinates'"""


@dataclass
class Scenario__RatioAverageSpeedToSpeedLimit(NumberData):
    """The average ratio between speed and speed limit over all road-based traffic participants and all frames in the file.

    In case no speed limit is present, the denominator is set to the maximum available speed limit of
    the country (e.g. 130 km/h for Germany) and 20 km/h is added to it (resulting in 150 km/h for Germany).
    """

    val: float = field(
        default_factory=lambda: no_default(field="Scenario__RatioAverageSpeedToSpeedLimit.val"), metadata=required
    )
    """The average ratio between speed and speed limit over all road-based traffic participants and all frames in the file."""

    name: Literal["scenario/ratio_average_speed_to_speed_limit"] = field(default="scenario/ratio_average_speed_to_speed_limit")
    """Is always 'scenario/ratio_average_speed_to_speed_limit'"""


@dataclass
class Scenario__RatioAverageSpeedToSpeedLimit__UStdDev(NumberData):
    """Uncertainty for `Scenario__RatioAverageSpeedToSpeedLimit` as standard deviation around the parent indicated value."""

    val: float = field(
        default_factory=lambda: no_default(field="Scenario__RatioAverageSpeedToSpeedLimit__UStdDev.val"), metadata=required
    )
    """Standard deviation around the parent indicated value. """

    name: Literal["scenario/ratio_average_speed_to_speed_limit/ustddev"] = field(
        default="scenario/ratio_average_speed_to_speed_limit/ustddev"
    )
    """Is always 'scenario/ratio_average_speed_to_speed_limit/ustddev'."""


@dataclass
class Scenario__WeekdayNumber(NumberData):
    """Weekday number according to ISO 8601."""

    val: int = field(default_factory=lambda: no_default(field="Scenario__WeekdayNumber.val"), metadata=required)
    """Integer Weekday number (Monday=1, Sunday = 7)."""

    name: Literal["scenario/weekday_number"] = field(default="scenario/weekday_number")
    """Is always 'scenario/weekday_number'"""



"""Status of the lights on the vehicle"""

# Copyright © 2024 understandAI GmbH
#
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files
# (the “Software”), to deal in the Software without restriction, including without limitation the rights to use, copy, modify,
# merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


from dataclasses import dataclass, field
from typing import Literal

from apischema.metadata import required
from uai_openlabel import (
    BooleanData,
    no_default,
)


@dataclass
class Lights__Brake(BooleanData):
    """Indicates whether the brake lights are on."""

    val: bool = field(default_factory=lambda: no_default(field="Lights__Brake.val"), metadata=required)
    """True iff the brake lights are on."""

    name: Literal["lights/brake"] = field(default="lights/brake")
    """Is always 'lights/brake'"""


@dataclass
class Lights__Indicator__Left(BooleanData):
    """
    Specifies whether the left turning signal indicator light is set or not.
    This attribute follows the value of the indicator light switch and not the light itself,
    i.e. this attribute stays True throughout the entire blinking process.
    """

    val: bool = field(default_factory=lambda: no_default(field="Lights__Indicator__Left.val"), metadata=required)
    """True iff the left indicator light switch is on."""

    name: Literal["lights/indicator/left"] = field(default="lights/indicator/left")
    """Is always 'lights/indicator/left'"""


@dataclass
class Lights__Indicator__Right(BooleanData):
    """
    Specifies whether the right turning signal indicator light is set or not.
    This attribute follows the value of the indicator light switch and not the light itself,
    i.e. this attribute stays True throughout the entire blinking process.
    """

    val: bool = field(default_factory=lambda: no_default(field="Lights__Indicator__Right.val"), metadata=required)
    """True iff the right indicator light switch is on."""

    name: Literal["lights/indicator/right"] = field(default="lights/indicator/right")
    """Is always 'lights/indicator/right'"""


@dataclass
class Lights__Front(BooleanData):
    """Specifies whether the front lights are on or off."""

    val: bool = field(default_factory=lambda: no_default(field="Lights__Front.val"), metadata=required)
    """True iff the front lights are on."""

    name: Literal["lights/front"] = field(default="lights/front")
    """Is always 'lights/front'"""


@dataclass
class Lights__Daytime(BooleanData):
    """Specifies whether the daytime lights are on or off."""

    val: bool = field(default_factory=lambda: no_default(field="Lights__Daytime.val"), metadata=required)
    """True iff the daytime lights are on."""

    name: Literal["lights/daytime"] = field(default="lights/daytime")
    """Is always 'lights/daytime'"""


@dataclass
class Lights__HighBeam(BooleanData):
    """Specifies whether the high-beam lights are on or off."""

    val: bool = field(default_factory=lambda: no_default(field="Lights__HighBeam.val"), metadata=required)
    """True iff the high beam lights are on."""

    name: Literal["lights/high_beam"] = field(default="lights/high_beam")
    """Is always 'lights/high_beam'"""



"""The `Event` class and its attributes

The `Event` class contains `EventData` which provides information about the event.
"""

# Copyright © 2024 understandAI GmbH
#
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files
# (the “Software”), to deal in the Software without restriction, including without limitation the rights to use, copy, modify,
# merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


from dataclasses import dataclass, field
from enum import Enum
from typing import Literal

from apischema.metadata import required
from uai_openlabel import (
    Attributes,
    FrameInterval,
    ObjectUid,
    TextData,
    TextType,
    VectorData,
    VectorType,
    no_default,
)
from uai_openlabel import (
    Event as BaseEvent,
)


@dataclass
class RoleAParticipantID(TextData):
    """Participant ID of traffic participant with Role A. Role A is dependent on the event type.

    lane change :       vehicle performing the lane change
    parking     :       vehicle performing the parking maneuver
    turning     :       vehicle that is turning/crossing
    overtaking  :       vehicle that is overtaking
    following   :       vehicle that is following

    """

    val: ObjectUid = field(default_factory=lambda: no_default(field="RoleAParticipantID.val"), metadata=required)
    """The ID of the traffic participant with Role A. This is the same ID as used for identifying the object. """

    name: Literal["event_participant/role_a_id"] = field(default="event_participant/role_a_id")
    """Is always 'event_participant/role_a_id'"""

    type: Literal[TextType.Value] = field(default=TextType.Value)
    """Is always 'value'"""


@dataclass
class RoleBParticipantIDs(VectorData):
    """Participant IDs of traffic participants with Role B. Role B is dependent on the event type.

    lane change :       empty
    parking     :       vehicles passing the Role A vehicle during the parking event and using the lane closest to the Role A vehicle
    turning     :       vehicles being within the scene during the turning event of the Role A vehicle and whose line of movement crosses the one of the Role A vehicle
    overtaking  :       vehicles that are being overtaken by the Role A vehicle
    following   :       vehicle that is being followed by the Role A vehicle (the vehicle immediately in front)

    """

    val: list[ObjectUid] = field(default_factory=lambda: no_default(field="RoleBParticipantIDs.val"), metadata=required)
    """The IDs of the traffic participant with Role B. This is the same ID as used for identifying the object. """

    name: Literal["event_participants/role_b_ids"] = field(default="event_participants/role_b_ids")
    """Is always 'event_participants/role_b_ids'"""

    type: Literal[VectorType.Values] = field(default=VectorType.Values)
    """Is always 'values'"""


@dataclass
class EventData(Attributes):
    """
    This class contains the attributes of the `Event` class.

    The attributes can be retrieved from the `text` and `vec` fields according to their type.
    """

    text: list[RoleAParticipantID] = field(default_factory=lambda: no_default(field="EventData.text"), metadata=required)
    """A list containing a single attribute, an instance of `RoleAParticipantID`."""

    vec: list[RoleBParticipantIDs] = field(default_factory=lambda: no_default(field="EventData.vec"), metadata=required)
    """A list containing a single attribute, an instance of `RoleBParticipantIDs`."""


class EventTypeValue(str, Enum):
    """Possible values of `Event.type`."""

    LANE_CHANGE = "lane change"
    """Lane change maneuver."""

    PARKING_MANEUVER = "parking"
    """Parking maneuver."""

    TURNING = "turning"
    """Turning maneuver."""

    OVERTAKING = "overtaking"
    """Overtaking maneuver."""

    FOLLOWING = "following"
    """Following maneuver."""


@dataclass
class Event(BaseEvent):
    """An event in the recording

    An event is an instantaneous situation that happens without a temporal interval.
    Events complement actions providing a mechanism to specify triggers or to connect actions and objects with causality relations.
    """

    event_data: EventData = field(default_factory=lambda: no_default(field="Event.event_data"), metadata=required)
    """Additional data to describe attributes of the event."""

    name: str = field(default_factory=lambda: no_default(field="Event.name"), metadata=required)
    """Name of the event, equal to type of event concatenated with ID of event. """

    frame_intervals: list[FrameInterval] = field(
        default_factory=lambda: no_default(field="Event.frame_intervals"), metadata=required
    )
    """The frame interval during which the event takes place. 
    For a lane change, the start and end frame are the same because we annotate the point in time when the center 
    of a vehicle is over the lane marking crossed during the lane change.
    """

    type: EventTypeValue = field(default_factory=lambda: no_default(field="Event.type"), metadata=required)
    """The type of the event."""



from typing import TypeVar

# Copyright © 2024 understandAI GmbH
#
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files
# (the “Software”), to deal in the Software without restriction, including without limitation the rights to use, copy, modify,
# merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


K = TypeVar("K")
V = TypeVar("V")


class ValidationError(ValueError):
    ...


def validate_dict_keys_and_value_types(dict_to_validate: dict[K, V], validation_template: dict[K, type[V]]) -> None:
    extra_keys_in_dict = set(dict_to_validate.keys()) - set(validation_template.keys())
    if len(extra_keys_in_dict):
        raise ValidationError(f"The dict to validate has the extra keys {extra_keys_in_dict}")

    missing_keys_from_template = set(validation_template.keys()) - set(dict_to_validate.keys())
    if len(missing_keys_from_template):
        raise ValidationError(f"The dict to validate is missing the keys {missing_keys_from_template}")

    for key in validation_template:
        if not isinstance(dict_to_validate[key], validation_template[key]):
            raise ValidationError(
                f"The dict entry under key {key} should be of type {validation_template[key]} "
                f"but is of type {dict_to_validate[key].__class__.__name__}"
            )



"""Information on the human operator"""

# Copyright © 2024 understandAI GmbH
#
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files
# (the “Software”), to deal in the Software without restriction, including without limitation the rights to use, copy, modify,
# merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


from dataclasses import dataclass, field
from enum import Enum
from typing import Literal

from apischema.metadata import required
from uai_openlabel import (
    NumberData,
    ObjectUid,
    TextData,
    VectorData,
    no_default,
)


@dataclass
class Operator__HeadRotation(NumberData):
    """Angle between head direction and forward direction of the vehicle in (rad), left=positive."""

    val: float = field(default_factory=lambda: no_default(field="Operator__HeadRotation.val"), metadata=required)
    """Angle between head direction and forward direction of the vehicle in (rad), left=positive."""

    name: Literal["operator/head_rotation"] = field(default="operator/head_rotation")
    """Is always 'operator/head_rotation'"""


@dataclass
class Operator__HeadRotation__UStdDev(NumberData):
    """Uncertainty for `Operator__HeadRotation` as standard deviation around the parent indicated value."""

    val: float = field(default_factory=lambda: no_default(field="Operator__HeadRotation__UStdDev.val"), metadata=required)
    """Standard deviation around the parent indicated value. """

    name: Literal["operator/head_rotation/ustddev"] = field(default="operator/head_rotation/ustddev")
    """Is always 'operator/head_rotation/ustddev'."""


@dataclass
class Operator__FocussedPoint(VectorData):
    """
    Tuple of the viewed (x,y,z) coordinates in vehicle coordinates of the object that the operator is focussing on.
    Is defined by extending the viewing beam from the operators' eyes into the surrounding 3D world to derive the target point.
    """

    val: tuple[float, float, float] = field(
        default_factory=lambda: no_default(field="Operator__FocussedPoint.val"), metadata=required
    )
    """The (x,y,z) coordinates of the focussed point."""

    name: Literal["operator/focussed_point"] = field(default="operator/focussed_point")
    """Is always 'operator/focussed_point'"""


@dataclass
class Operator__FocussedPoint__UStdDev(VectorData):
    """Uncertainty for `Operator__FocussedPoint` as standard deviation around the parent indicated value."""

    val: tuple[float, float, float] = field(
        default_factory=lambda: no_default(field="Operator__FocussedPoint__UStdDev.val"), metadata=required
    )
    """Standard deviation around the parent indicated value. """

    name: Literal["operator/focussed_point/ustddev"] = field(default="operator/focussed_point/ustddev")
    """Is always 'operator/focussed_point/ustddev'."""


@dataclass
class Operator__ViewingAngle(NumberData):
    """Drivers eye viewing angle as the angle between the top edge of the hood and the horizon, in (rad)."""

    val: float = field(default_factory=lambda: no_default(field="Operator__ViewingAngle.val"), metadata=required)
    """Drivers eye viewing angle as the angle between the top edge of the hood and the horizon, in (rad)."""

    name: Literal["operator/viewing_angle"] = field(default="operator/viewing_angle")
    """Is always 'operator/viewing_angle'"""


class Operator__FocussedObjectValue(str, Enum):
    """Possible values of `Operator__FocussedObject`"""

    LEADING_VEHICLE = "leading_vehicle"
    """Focussed object is the vehicle in front of the operator"""

    COLLIDING_VEHICLE = "colliding_vehicle"
    """Focussed object is a vehicle in the scene having a collision"""

    GENERIC_VEHICLE = "generic_vehicle"
    """Focussed object is a generic vehicle in the scene"""

    SIGNAL = "signal"
    """Focussed object is a traffic signal"""

    CENTER_DISPLAY = "center_display"
    """Focussed object is the vehicle-inter center display"""

    INSTRUMENT_CLUSTER = "instrument_cluster"
    """Focussed object is the vehicle-inter instrument cluster"""

    HUD = "hud"
    """Focussed object is the vehicle-inter head-up display"""

    OTHER_DISPLAY = "other_display"
    """Focussed object is another vehicle-inter head-up display"""

    LEFT_MIRROR = "left_mirror"
    """Focussed object is the left rear-view mirror"""

    RIGHT_MIRROR = "right_mirror"
    """Focussed object is the right rear-view mirror"""

    OTHER_INTERIOR = "other_interior"
    """Focussed object is another object in the vehicle's interior (or on the vehicle's body)"""

    OTHER_EXTERIOR = "other_exterior"
    """Focussed object is another exterior object"""


@dataclass
class Operator__FocussedObject(TextData):
    """Object the operator is focussing on."""

    val: Operator__FocussedObjectValue = field(
        default_factory=lambda: no_default(field="Operator__FocussedObject.val"), metadata=required
    )
    """See `Operator__FocussedObjectValue` for possible values. """

    name: Literal["operator/focussed_object"] = field(default="operator/focussed_object")
    """Is always 'operator/focussed_object'"""


@dataclass
class Operator__FocussedObject__Uncertainties(VectorData):
    """
    Uncertainty values for `Operator__FocussedObject` (must accumulate to 1)
    """

    val: tuple[float, float, float, float, float, float, float, float, float, float, float, float, float] = field(
        default_factory=lambda: no_default(field="Operator__FocussedObject__Uncertainties.val"), metadata=required
    )
    """Tuple with class probabilities (must accumulate to 1), in the following order: 
     - leading_vehicle
     - colliding_vehicle
     - generic_vehicle
     - signal
     - center_display
     - instrument_cluster
     - other_display
     - hud
     - other_display
     - left_mirror
     - right_mirror
     - other_interior
     - other_exterior
    """

    name: Literal["operator/focussed_object/uncertainties"] = field(default="operator/focussed_object/uncertainties")
    """Is always 'operator/focussed_object/uncertainties'"""


@dataclass
class Operator__FocussedObject__Id(TextData):
    """
    ID of the object the operator is focussing on. Can be used to index the `AveasOpenLabel.objects` field.
    Is only provided if the object the operator is focussing on is indexed in this file.
    """

    val: ObjectUid = field(default_factory=lambda: no_default(field="Operator__FocussedObject__Id.val"), metadata=required)
    """ID of the focussed object."""

    name: Literal["operator/focussed_object/id"] = field(default="operator/focussed_object/id")
    """Is always 'operator/focussed_object/id'"""


@dataclass
class Operator__Pupil(VectorData):
    """Tuple of pupil-specific information for the operator"""

    val: tuple[float, float, float, float, float, float, float] = field(
        default_factory=lambda: no_default(field="Operator__Pupil.val"), metadata=required
    )
    """Tuple with information on:
    1. left pupil diameter in (mm)
    2. right pupil diameter in (mm)
    3. left iris diameter in (mm)
    4. right iris diameter in (mm)
    5. left openness range
    6. right openness range
    7. focus stability range"""
    # TODO/JZ: Was bedeutet "openness range"

    name: Literal["operator/pupil"] = field(default="operator/pupil")
    """Is always 'operator/pupil'"""


@dataclass
class Operator__Pupil__UStdDev(VectorData):
    """Uncertainty for `Operator__Pupil` as standard deviation around the parent indicated value."""

    val: tuple[float, float, float, float, float, float, float] = field(
        default_factory=lambda: no_default(field="Operator__Pupil__UStdDev.val"), metadata=required
    )
    """Standard deviation around the parent indicated value. """

    name: Literal["operator/pupil/ustddev"] = field(default="operator/pupil/ustddev")
    """Is always 'operator/pupil/ustddev'."""


class Operator__HandInteractionAreaValue(str, Enum):
    """Possible values of `Operator__HandInteractionArea`."""

    STEERING_WHEEL = "steering_wheel"
    """Hand is on steering wheel"""

    CENTER_DISPLAY = "center_display"
    """Hand is on center display"""

    ARMREST = "armrest"
    """Hand is on armrest"""

    OTHER = "other"
    """Hand is on other element"""


@dataclass
class Operator__HandInteractionArea(VectorData):
    """Tuple of the left and right hand interaction area. See `Operator__HandInteractionAreaValue` for possible values."""

    val: tuple[Operator__HandInteractionAreaValue, Operator__HandInteractionAreaValue] = field(
        default_factory=lambda: no_default(field="Operator__HandInteractionArea.val"), metadata=required
    )
    """ Tuple of the left and right hand interaction area"""

    name: Literal["operator/hand_interaction_area"] = field(default="operator/hand_interaction_area")
    """Is always 'operator/hand_interaction_area'"""


@dataclass
class Operator__SixDoFRotationAndAcceleration(VectorData):
    """For simulation studies: Current parameters to a six-degrees-of-freedom physical driving simulator, if used.

    This can be used to evaluate the settings of a suitable physical driving simulator (e.g. a hexapod),
    which is the seat of the driving simulator and can exert forces onto the participant. For physical limitations,
    these values may differ from the actual physical properties of the simulated vehicle.
    """

    val: tuple[float, float, float, float, float, float] = field(
        default_factory=lambda: no_default(field="Operator__SixDoFRotationAndAcceleration.val"), metadata=required
    )
    """Tuple with information: 
     - x in (m/s^2): Acceleration in x direction.
     - y in (m/s^2): Acceleration in y direction.
     - z in (m/s^2): Acceleration in z direction.
     - rx in (rad): Euler roll angle (positive: right roll).
     - ry in (rad): Euler pitch angles (positive: pitch down).
     - rz in (rad): Euler yaw angles (positive: yaw left).
    """

    name: Literal["operator/six_dof_rotation_and_acceleration"] = field(default="operator/six_dof_rotation_and_acceleration")
    """Is always 'operator/six_dof_rotation_and_acceleration'"""


@dataclass
class Operator__Age(NumberData):
    """Age of the vehicle's operator in years"""

    val: int = field(default_factory=lambda: no_default(field="Operator__Age.val"), metadata=required)
    """Age of the vehicle's operator in years"""

    name: Literal["operator/age"] = field(default="operator/age")
    """Is always 'operator/age'"""


class Operator__GenderValue(str, Enum):
    """Possible values of `Operator__Gender`."""

    MALE = "male"
    """male operator"""

    FEMALE = "female"
    """female operator"""

    DIVERSE = "diverse"
    """diverse operator"""


@dataclass
class Operator__Gender(TextData):
    """Gender of the vehicle operator"""

    val: Operator__GenderValue = field(default_factory=lambda: no_default(field="Operator__Gender.val"), metadata=required)
    """See `Operator__GenderValue` for possible values."""

    name: Literal["operator/gender"] = field(default="operator/gender")
    """Is always 'operator/gender'"""


@dataclass
class Operator__Personality(
    VectorData
):  # TODO: Wenn es drin bleiben soll, brauchen wir Quellen / Berechnungsvorschriften für alle diese Felder.
    """Tuple of the vehicle operator's personality scores"""

    val: tuple[float, float, float, float, float, float, float, float, float, float] = field(
        default_factory=lambda: no_default(field="Operator__Personality.val"), metadata=required
    )
    """
    tuple with information: 
     - openness score (lowerCI,upperCI) 
     - conscientiousness score (lowerCI,upperCI) 
     - extraversion score (lowerCI,upperCI) 
     - agreeableness score (lowerCI,upperCI) 
     - neuroticism score (lowerCI,upperCI)
     """

    name: Literal["operator/personality"] = field(default="operator/personality")
    """Is always 'operator/personality'"""


@dataclass
class Operator__BodyHeight(NumberData):
    """Body height of the vehicle operator in (m)"""

    val: float = field(default_factory=lambda: no_default(field="Operator__BodyHeight.val"), metadata=required)
    """Body height of the vehicle operator in (m)"""

    name: Literal["operator/body_height"] = field(default="operator/body_height")
    """Is always 'operator/body_height'"""



"""The `Frame` class and its attributes
"""

# Copyright © 2024 understandAI GmbH
#
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files
# (the “Software”), to deal in the Software without restriction, including without limitation the rights to use, copy, modify,
# merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


from dataclasses import dataclass, field
from typing import Optional, Union

from uai_openlabel import (
    Frame as BaseFrame,
)
from uai_openlabel import (
    FrameProperties as BaseFrameProperties,
)
from uai_openlabel import (
    Number,
    ObjectInFrame,
    ObjectUid,
)


@dataclass
class FrameProperties(BaseFrameProperties):
    """This object contains metadata about this frame."""

    timestamp: Optional[Union[str, Number]] = field(default=None)
    """The timestamp indicates a time instant as a numerical value to describe this frame."""


@dataclass
class Frame(BaseFrame):
    """A frame is a container of dynamic, timewise, information."""

    frame_properties: Optional[FrameProperties] = field(default=None)
    """This field contains information about this frame."""

    objects: Optional[dict[ObjectUid, ObjectInFrame]] = field(default=None)
    """
    This field contains dynamic information on OpenLABEL objects. Object values contain an "object_data" field.
    Dict keys are unique per object in the scenario. 
    Static attributes of an object can be found in `AveasOpenLabel.objects`.
    """



"""Contains the `Metadata` class
"""

# Copyright © 2024 understandAI GmbH
#
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files
# (the “Software”), to deal in the Software without restriction, including without limitation the rights to use, copy, modify,
# merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


from dataclasses import dataclass, field
from enum import Enum
from typing import Literal

from apischema.metadata import required
from uai_openlabel import (
    Metadata as BaseMetadata,
)
from uai_openlabel import (
    no_default,
)


class RightOfUse(str, Enum):
    """Possible values of 'right_of_use'."""

    COMMERCIAL = "commercial"
    """Commercial use of data allowed. Implying also usage for research purposes."""

    RESEARCH_ONLY = "research_only"
    """Data can only be used for research purposes."""


class AcquisitionMethod(str, Enum):
    """Possible values of 'acquisition_method'."""

    AERIAL = "aerial"
    """Aerial acquisition with plane."""

    STATIONARY = "stationary"
    """Stationary data acquisition, e.g. with infrastructure sensors."""

    IN_VEHICLE = "in-vehicle"
    """Data acquisition with moving recording vehicle."""


@dataclass
class Metadata(BaseMetadata):
    """This JSON object contains metadata about the annotation file itself."""

    aveas_schema_version: Literal["0.4.14"] = field(default="0.4.14")
    """The version of the aveas_openlabel library used to generate this file."""

    right_of_use: RightOfUse = field(default_factory=lambda: no_default(field="Metadata.right_of_use"), metadata=required)
    """Specifies the usage rights of the data."""

    acquisition_method: AcquisitionMethod = field(
        default_factory=lambda: no_default(field="Metadata.acquisition_method"), metadata=required
    )
    """Specifies the acquisition method used for obtaining the data in this file."""

    acquisition_partner: str = field(
        default_factory=lambda: no_default(field="Metadata.acquisition_partner"), metadata=required
    )
    """
    Institution or company involved in recording the data in this file. 
    This is a free-text field that may contain copyright information. 
    """

    acquisition_date: str = field(default_factory=lambda: no_default(field="Metadata.acquisition_date"), metadata=required)
    """
    The date on which the data were acquired in the format yyyy-MM-ddTHH:mm:ss.FFFZ. 
    Here, 'T' is used as time designator. '.' is used as separator for the following millisecond portion. 

    - yyyy: Year (four digits), ex. 2021
    - MM: Month with leading zero, ex. 09
    - dd: Day in month with leading zero
    - mm: Minutes with leading zero
    - ss: seconds with leading zero
    - FFF: Milliseconds with leading zeros
    - Z: 'Z' if the time zone is UTC, '±[hh]:[mm]', '±[hh][mm]', or '±[hh]' otherwise, ex. '+0100'
    """

    projection_string: str = field(default_factory=lambda: no_default(field="Metadata.projection_string"), metadata=required)
    """
    The geographic reference system used for the coordinates in this OpenLABEL-file and in corresponding OpenDRIVE-files. 
    Projection strings follow official parameter sets for proj-strings from EPSG.
    """

    threshold_gttc: float = field(default_factory=lambda: no_default(field="Metadata.threshold_gttc"), metadata=required)
    """
    Traffic participants with a geometrical time-to-collision (gTTC) value lower than this threshold will appear in 
    the `Impact__gTTC__ObjectIds` and `Impact__gTTC__Values` attributes of this file. 
    """

    threshold_pret: float = field(default_factory=lambda: no_default(field="Metadata.threshold_pret"), metadata=required)
    """
    Traffic participants with a predicted encroachment time (PrET) value lower than this threshold will appear in 
    the `Impact__PrET__ObjectIds` and `Impact__PrET__Values` attributes of this file. 
    """

    threshold_thw: float = field(default_factory=lambda: no_default(field="Metadata.threshold_thw"), metadata=required)
    """
    Traffic participants with a time headway (THW) value lower than this threshold will appear in 
    the `Impact__THW__ObjectIds` and `Impact__THW__Values` attributes of this file. 
    """



"""HMI feedback information"""

# Copyright © 2024 understandAI GmbH
#
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files
# (the “Software”), to deal in the Software without restriction, including without limitation the rights to use, copy, modify,
# merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


from dataclasses import dataclass, field
from typing import Literal

from apischema.metadata import required
from uai_openlabel import (
    NumberData,
    no_default,
)


@dataclass
class HmiFeedback__Visual(NumberData):
    """Numeric ID indicating some study-specific visual feedback given to the operator, e.g. via a warning light.
    A value of 0 means that no relevant visual feedback was given to the driver.
    Any value <> 0 denotes a study-specific feedback ID that must be specified in the study description document
    referenced in the resources field.

    See also:
    - `HmiFeedback__Acoustic`
    - `HmiFeedback__Other`
    """

    val: int = field(default_factory=lambda: no_default(field="HmiFeedback__Visual.val"), metadata=required)
    """Numeric ID of the study-specific visual feedback, or 0 if no visual feedback."""

    name: Literal["hmi_feedback/visual"] = field(default="hmi_feedback/visual")
    """Is always 'hmi_feedback/visual'"""


@dataclass
class HmiFeedback__Acoustic(NumberData):
    """Numeric ID indicating some study-specific acoustic feedback given to the operator, e.g. via a warning sound.
    A value of 0 means that no relevant acoustic feedback was given to the driver.
    Any value <> 0 denotes a study-specific feedback ID that must be specified in the study description document
    referenced in the resources field.

    See also:
    - `HmiFeedback__Visual`
    - `HmiFeedback__Other`
    """

    val: int = field(default_factory=lambda: no_default(field="HmiFeedback__Acoustic.val"), metadata=required)
    """Numeric ID of the study-specific acoustic feedback, or 0 if no acoustic feedback"""

    name: Literal["hmi_feedback/acoustic"] = field(default="hmi_feedback/acoustic")
    """Is always 'hmi_feedback/acoustic'"""


@dataclass
class HmiFeedback__Other(NumberData):
    """Numeric ID indicating some study-specific other feedback given to the operator, e.g. haptic feedback.
    A value of 0 means that no relevant other feedback was given to the driver.
    Any value <> 0 denotes a study-specific feedback ID that must be specified in the study description document
    referenced in the resources field.

    See also:
    - `HmiFeedback__Visual`
    - `HmiFeedback__Acoustic`
    """

    val: int = field(default_factory=lambda: no_default(field="HmiFeedback__Other.val"), metadata=required)
    """Numeric ID of the study-specific other feedback, or 0 if no other feedback"""

    name: Literal["hmi_feedback/other"] = field(default="hmi_feedback/other")
    """Is always 'hmi_feedback/other'"""
